/** 
 *  Odor Analytics Source File
 *
 *  Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
 */

#ifndef TMatrix_h
#define TMatrix_h

#include <stdio.h>
#include "stoolkit/SIO.h"

#define FType double
//#define FType float

class TMatrix
{
public:
  TMatrix (const SFile& file, unsigned int rows, unsigned int cols);
  TMatrix (const SFile& file);

  TMatrix (const TMatrix& m);
  TMatrix& operator=(const TMatrix& m);

  bool print (FILE* output);
  bool scan (FILE* input);

  TMatrix pow (const SFile& file,  FType type) const;
  TMatrix add (const SFile& file,  FType type) const;
  TMatrix log (const SFile& file) const;
  TMatrix exp (const SFile& file) const;
  TMatrix abs (const SFile& file) const;

  TMatrix de (const SFile& file, const TMatrix& matrix1) const;
  TMatrix pe (const SFile& file, const TMatrix& matrix1) const;

  TMatrix pm (const SFile& file, const TMatrix& matrix1) const;
  TMatrix ps (const SFile& file, FType type) const;

  TMatrix se (const SFile& file, const TMatrix& matrix1) const;
  TMatrix me (const SFile& file, const TMatrix& matrix1) const;

  TMatrix tr (const SFile& file) const;
  TMatrix sort (const SFile& file, const SFile& indexfile, 
       unsigned int index, bool byroow) const;

  static TMatrix unit (const SFile& file, unsigned int rowcols);
  static TMatrix zero (const SFile& file, unsigned int row, unsigned int col);
  static TMatrix rand100 (const SFile& file, unsigned int row, unsigned int col);


  SString getError() const;

  inline unsigned int rows() const;
  inline unsigned int columns() const;
  inline bool isOK() const;

  inline void set (unsigned int r,  unsigned int c, FType vle);
  inline FType get (unsigned int r,  unsigned int c) const;

private:
  SFile        file;
  SFileImage   image;
  SString      error;

  unsigned int rcount;
  unsigned int ccount;
  FType        *darray;

  void sort (unsigned int* index, FType* key, int from, int to) const;
};

bool 
TMatrix::isOK() const
{
  return (darray!=0 && ccount !=0 && rcount !=0);
}

FType
TMatrix::get (unsigned int r, unsigned int c) const
{
  return darray[ccount*r+c];
}

void 
TMatrix::set (unsigned int r, unsigned int c, FType vle)
{
  darray[ccount*r+c] = vle;
}

unsigned int 
TMatrix::rows() const
{
  return rcount;
}

unsigned int 
TMatrix::columns() const
{
  return ccount;
}

#endif /* TMatrix_h */
